/*requires Schema.Common.sql*/
/*requires Table.Common.CommandLog.sql*/


-- !!! Modified default value for @LogSchemaName to 'Common' instead of 'dbo'

DECLARE @ProcedureSchema NVARCHAR(256);
DECLARE @ProcedureName   NVARCHAR(256);

SET @ProcedureSchema = 'Common' ;
SET @ProcedureName = 'CommandExecute' ;

PRINT '--------------------------------------------------------------------------------------------------------------';
PRINT 'PROCEDURE [' + @ProcedureSchema + '].[' + @ProcedureName +  ']';

IF  NOT EXISTS (SELECT 1 FROM sys.objects WHERE object_id = OBJECT_ID(N'[' + @ProcedureSchema + N'].[' + @ProcedureName +  N']') AND type in (N'P'))
BEGIN
    BEGIN TRY
        EXECUTE ('CREATE Procedure [' + @ProcedureSchema + '].[' + @ProcedureName +  '] ( ' +
                ' @ServerName    varchar(512), ' +
                ' @DbName    varchar(50) ' +
                ') ' +
                'AS ' +
                'BEGIN ' +
                '   SELECT ''Not implemented'' ' +
                'END')
    END TRY
    BEGIN CATCH
        PRINT '   Error while trying to create procedure'
        RETURN
    END CATCH

    PRINT '   PROCEDURE created.'
END
GO

ALTER PROCEDURE [Common].[CommandExecute] (
    @Command                        nvarchar(max),
    @CommandType                    nvarchar(max),
    @RunMode                        int,
    @Comments                       nvarchar(max)   = NULL,
    @DatabaseName                   nvarchar(max)   = NULL,
    @SchemaName                     nvarchar(max)   = NULL,
    @ObjectName                     nvarchar(max)   = NULL,
    @ObjectType                     nvarchar(max)   = NULL,
    @IndexName                      nvarchar(max)   = NULL,
    @IndexType                      int             = NULL,
    @StatisticsName                 nvarchar(max)   = NULL,
    @PartitionNumber                int             = NULL,
    @ExtendedInfo                   xml             = NULL,
    @LogSchemaName                  VARCHAR(256)    = 'Common',
    @LogToTable                     char(1), --previously nvarchar(max) but is used for Y|N
    @LogTableName                   VARCHAR(256)    = 'CommandLog',
    @RunCommand                     char(1),  --previously nvarchar(max) but is used for Y|N
    @Debug                          BIT             = 0
)
AS
/*
  ===================================================================================
   DESCRIPTION:

   PARAMETERS:

   REQUIREMENTS:

   EXAMPLE USAGE :

        exec [Common].[CommandExecute]
                @Command        = 'SELECT GETDATE()',
                @CommandType    = 'Test',
                @RunMode        = 2,
                @LogToTable     = 'y',
                @RunCommand     = 'y',
                @LogSchemaName  = 'Common',
                @Debug          = 1
        ;

        select * from [Common].CommandLog;

   Original Source: https://ola.hallengren.com
*/
BEGIN
    SET NOCOUNT ON;

    DECLARE @LineFeed               CHAR(2);
    DECLARE @tsql                   nvarchar(max);
    DECLARE @StartMessage           nvarchar(max);
    DECLARE @EndMessage             nvarchar(max);
    DECLARE @ErrorMessage           nvarchar(max);
    DECLARE @ErrorMessageOriginal   nvarchar(max);

    DECLARE @StartTime              datetime;
    DECLARE @EndTime                datetime;

    DECLARE @StartTimeSec           datetime;
    DECLARE @EndTimeSec             datetime;

    DECLARE @CommandID              int;

    DECLARE @ErrorId                int;
    DECLARE @ReturnCode             int;

    DECLARE @LogTableFullName       VARCHAR(max);

    SELECT
        @LineFeed           = CHAR(13) + CHAR(10),
        @tsql               = '',
        @LogSchemaName      = ISNULL(@LogSchemaName,'dbo'),
        @LogTableName       = ISNULL(@LogTableName,'CommandLog')
    ;

    SET @ErrorId      = 0;
    SET @ReturnCode   = 0;
    SET @LogToTable   = UPPER(@LogToTable);
    SET @RunCommand   = UPPER(@RunCommand);

    -- logging table
    SET @LogTableFullName   = QUOTENAME(@LogSchemaName) + '.' + QUOTENAME(@LogTableName);

    if (@Debug = 1)
    BEGIN
        PRINT 'Checking core requirements';
    END;

    IF (@LogToTable = 'Y' AND OBJECT_ID(@LogTableFullName) IS NULL)
    BEGIN
        SET @ErrorMessage = 'The table ' + @LogTableFullName + ' is missing.' + @LineFeed + ' ';
        RAISERROR(@ErrorMessage,16,1) WITH NOWAIT;
        SET @ErrorId = @@ERROR;
    END;

    IF (@ErrorId <> 0)
    BEGIN
        SET @ReturnCode = @ErrorId;
        GOTO ReturnCode;
    END;

    if (@Debug = 1)
    BEGIN
        PRINT 'Checking input parameters';
    END;

    IF (@Command IS NULL OR @Command = '')
    BEGIN
        SET @ErrorMessage = 'The value for the parameter @Command is not supported.' + @LineFeed + ' ';
        RAISERROR(@ErrorMessage,16,1) WITH NOWAIT;
        SET @ErrorId = @@ERROR;
    END;

    IF (@CommandType IS NULL OR @CommandType = '' OR LEN(@CommandType) > 60)
    BEGIN
        SET @ErrorMessage = 'The value for the parameter @CommandType is not supported.' + @LineFeed + ' ';
        RAISERROR(@ErrorMessage,16,1) WITH NOWAIT;
        SET @ErrorId = @@ERROR;
    END;

    IF @RunMode NOT IN(1,2) OR @RunMode IS NULL
    BEGIN
        SET @ErrorMessage = 'The value for the parameter @RunMode is not supported.' + @LineFeed + ' ';
        RAISERROR(@ErrorMessage,16,1) WITH NOWAIT;
        SET @ErrorId = @@ERROR;
    END;

    IF (@LogToTable NOT IN('Y','N') OR @LogToTable IS NULL)
    BEGIN
        SET @ErrorMessage = 'The value for the parameter @LogToTable is not supported. Expected ''Y'' or ''N''.' + @LineFeed + ' ';
        RAISERROR(@ErrorMessage,16,1) WITH NOWAIT;
        SET @ErrorId = @@ERROR;
    END;

    IF (@RunCommand NOT IN('Y','N') OR @RunCommand IS NULL)
    BEGIN
        SET @ErrorMessage = 'The value for the parameter @RunCommand is not supported.' + @LineFeed + ' ';
        RAISERROR(@ErrorMessage,16,1) WITH NOWAIT;
        SET @ErrorId = @@ERROR;
    END;

    IF (@ErrorId <> 0)
    BEGIN
        SET @ReturnCode = @ErrorId;
        GOTO ReturnCode;
    END;

    if (@Debug = 1)
    BEGIN
        PRINT 'Getting Log initial information';
    END;

    SET @StartTime      = GETDATE();
    SET @StartTimeSec   = CONVERT(datetime,CONVERT(nvarchar,@StartTime,120),120);

    SET @StartMessage = 'Date and time: ' + CONVERT(nvarchar,@StartTimeSec,120) + @LineFeed;
    SET @StartMessage = @StartMessage + 'Command: ' + @Command;

    IF (@Comments IS NOT NULL)
    BEGIN
        SET @StartMessage = @StartMessage + @LineFeed + 'Comment: ' + @Comments;
    END;

    SET @StartMessage = REPLACE(@StartMessage,'%','%%');
    -- append to log
    RAISERROR(@StartMessage,10,1) WITH NOWAIT;

    IF (@LogToTable = 'Y')
    BEGIN

        SET @tsql = 'INSERT INTO ' + @LogTableFullName + '(' + @LineFeed +
                    '    DatabaseName, SchemaName, ObjectName, ObjectType, IndexName, IndexType, StatisticsName, PartitionNumber, ExtendedInfo, CommandType, Command, StartTime' + @LineFeed +
                    ')' + @LineFeed +
                    'VALUES (' + @LineFeed +
                    '    @DatabaseName, @SchemaName, @ObjectName, @ObjectType, @IndexName, @IndexType, @StatisticsName, @PartitionNumber, @ExtendedInfo, @CommandType, @Command, @StartTime' + @LineFeed +
                    ');'+ @LineFeed +
                    'SELECT @NewCommandID = SCOPE_IDENTITY()'
        ;

        exec sp_executesql
                @tsql,
                N'@DatabaseName nvarchar(max), @SchemaName nvarchar(max), @ObjectName nvarchar(max), @ObjectType nvarchar(max), @IndexName nvarchar(max), @IndexType int, @StatisticsName nvarchar(max), @PartitionNumber nvarchar(max), @ExtendedInfo xml, @CommandType nvarchar(max), @Command nvarchar(max), @StartTime datetime,@NewCommandID int OUTPUT',
                @DatabaseName       = @DatabaseName,
                @SchemaName         = @SchemaName,
                @ObjectName         = @ObjectName,
                @ObjectType         = @ObjectType,
                @IndexName          = @IndexName,
                @IndexType          = @IndexType,
                @StatisticsName     = @StatisticsName,
                @PartitionNumber    = @PartitionNumber,
                @ExtendedInfo       = @ExtendedInfo,
                @CommandType        = @CommandType,
                @Command            = @Command,
                @StartTime          = @StartTime,
                @NewCommandID       = @CommandID OUTPUT
       ;

    END;

    if (@Debug = 1)
    BEGIN
        PRINT 'Executing command';
    END;

    IF (@RunMode = 1 AND @RunCommand = 'Y')
    BEGIN
        EXECUTE(@Command);
        SET @ErrorId = @@ERROR;
        SET @ReturnCode = @ErrorId;
    END;

    IF (@RunMode = 2 AND @RunCommand = 'Y')
    BEGIN
        BEGIN TRY
            EXECUTE(@Command);
        END TRY
        BEGIN CATCH
            SET @ErrorId                = ERROR_NUMBER();
            SET @ReturnCode             = @ErrorId;
            SET @ErrorMessageOriginal   = ERROR_MESSAGE();
            SET @ErrorMessage           = 'Msg ' + CAST(@ErrorId AS nvarchar) + ', ' + ISNULL(@ErrorMessageOriginal,'');

            RAISERROR(@ErrorMessage,16,1) WITH NOWAIT;
        END CATCH

    END;

    if (@Debug = 1)
    BEGIN
        PRINT 'Logging completing information';
    END;


    SET @EndTime    = GETDATE();
    SET @EndTimeSec = CONVERT(datetime,CONVERT(varchar,@EndTime,120),120);

    SET @EndMessage = 'Outcome: ' + CASE WHEN @RunCommand = 'N' THEN 'Not Executed' WHEN @ErrorId = 0 THEN 'Succeeded' ELSE 'Failed' END + @LineFeed;
    SET @EndMessage = @EndMessage + 'Duration: ' + CASE WHEN DATEDIFF(ss,@StartTimeSec, @EndTimeSec)/(24*3600) > 0 THEN CAST(DATEDIFF(ss,@StartTimeSec, @EndTimeSec)/(24*3600) AS nvarchar) + '.' ELSE '' END + CONVERT(nvarchar,@EndTimeSec - @StartTimeSec,108) + @LineFeed;
    SET @EndMessage = @EndMessage + 'Date and time: ' + CONVERT(nvarchar,@EndTimeSec,120) + @LineFeed + ' ';
    SET @EndMessage = REPLACE(@EndMessage,'%','%%');
    RAISERROR(@EndMessage,10,1) WITH NOWAIT;

    IF (@LogToTable = 'Y')
    BEGIN
        SET @tsql = 'UPDATE ' + @LogTableFullName + @LineFeed +
                    'SET' + @LineFeed +
                    '    EndTime     = @EndTime,' + @LineFeed +
                    '    ErrorNumber = CASE WHEN @RunCommand = ''N'' THEN NULL ELSE @ErrorId END,' + @LineFeed +
                    '    ErrorText   = @ErrorMessageOriginal' + @LineFeed +
                    'WHERE CommandId = @CommandID'
        ;

         exec sp_executesql
                @tsql,
                N'@EndTime datetime,@RunCommand char(1),@ErrorId int,@ErrorMessageOriginal nvarchar(max), @CommandID int',
                @EndTime                = @EndTime,
                @RunCommand             = @RunCommand,
                @ErrorId                = @ErrorId,
                @ErrorMessageOriginal   = @ErrorMessageOriginal,
                @CommandID                     = @CommandID
        ;
    END;

    ReturnCode:
    IF (@ReturnCode <> 0)
    BEGIN
        RETURN @ReturnCode;
    END;

    if (@Debug = 1)
    BEGIN
        PRINT '-- Execution of [Common].[CommandExecute] completed.';
    END;
END
GO


IF (@@ERROR = 0)
BEGIN
    PRINT '   PROCEDURE altered.';
END
ELSE
BEGIN
    PRINT '   Error while trying to alter procedure';
    RETURN
END;
GO

PRINT '--------------------------------------------------------------------------------------------------------------';
PRINT '';
GO